package com.katalon.oauth2

import static com.kms.katalon.core.testobject.ObjectRepository.findTestObject

import org.apache.commons.lang3.StringUtils
import org.apache.http.client.utils.URIBuilder
import org.openqa.selenium.Cookie
import org.openqa.selenium.WebDriver
import org.openqa.selenium.WebElement
import org.openqa.selenium.support.ui.ExpectedConditions
import org.openqa.selenium.support.ui.WebDriverWait
import com.kms.katalon.core.annotation.Keyword
import com.kms.katalon.core.webui.driver.DriverFactory
import com.kms.katalon.core.webui.keyword.WebUiBuiltInKeywords as WebUI
import org.openqa.selenium.By

public class OAuth2Keywords {

	private static final String RESPONSE_TYPE_PARAM = "response_type";

	private static final String CLIENT_ID_PARAM = "client_id";

	private static final String STATE_PARAM = "state";

	private static final String REDIRECT_URI_PARAM = "redirect_uri";

	private static final String SCOPE_PARAM = "scope";

	private static final String ACCESS_TOKEN_PARAM = "access_token";

	private static final String AUTHORIZATION_PAGE_URL = "https://www.oauth.com/playground/auth-dialog.html";

	private static final String REDIRECT_URL = "https://www.oauth.com/playground/implicit.html";

	private static final String CLIENT_ID = "DNYWEt3HEeVYKMvlMsQ2_dyQ";

	@Keyword
	String getAccessTokenWithImplicitGrantType(String username, String password, String scope) {
		String state = generateStateParameter();
		String authorizationPageUrl = getAuthorizationPageUrl(state, scope);
		WebUI.openBrowser('');
		WebDriver driver = DriverFactory.getWebDriver();
		WebDriverWait wait = new WebDriverWait(driver, 5, 100L);

		WebUI.navigateToUrl(authorizationPageUrl);
		DriverFactory.getWebDriver().manage().addCookie(new Cookie("user_login", username));
		DriverFactory.getWebDriver().manage().addCookie(new Cookie("user_password", password));
		WebUI.navigateToUrl(authorizationPageUrl);
		WebUI.setText(findTestObject('Page_Sample Auth Dialog/input_Username_username'), username);
		WebUI.setText(findTestObject('Page_Sample Auth Dialog/input_Password_password'), password);
		WebUI.click(findTestObject('Page_Sample Auth Dialog/a_Log In'));

		WebUI.click(findTestObject('Object Repository/Page_Sample Auth Dialog/a_Approve'));

		wait.until(ExpectedConditions.urlContains("access_token"));

		String redirectUrl = WebUI.getUrl()
		if (validateRedirectUrl(redirectUrl, state)) {
			String accessToken = getFragmentParameter(redirectUrl, ACCESS_TOKEN_PARAM)
			return accessToken;
		}
		return null;
	}

	private String generateStateParameter() {
		return System.currentTimeMillis().toString();
	}

	private String getAuthorizationPageUrl(String state, String scope) {
		URIBuilder uriBuilder = new URIBuilder(AUTHORIZATION_PAGE_URL);
		uriBuilder.setParameter(RESPONSE_TYPE_PARAM, "token");
		uriBuilder.setParameter(CLIENT_ID_PARAM, CLIENT_ID);
		uriBuilder.setParameter(REDIRECT_URI_PARAM, REDIRECT_URL);
		uriBuilder.setParameter(SCOPE_PARAM, scope);
		uriBuilder.setParameter(STATE_PARAM, state);
		return uriBuilder.toString();
	}

	private String validateRedirectUrl(String url, String state) {
		String stateParameter = getFragmentParameter(url, STATE_PARAM);
		return StringUtils.isNotBlank(stateParameter) && StringUtils.equals(stateParameter, state);
	}

	private String getFragmentParameter(String url, String parameterName) {
		URIBuilder uriBuilder = new URIBuilder(url);
		String fragment = uriBuilder.getFragment();
		Map<String, String> fragmentParamMap = getFragmentParamMap(fragment);
		return fragmentParamMap.get(parameterName);
	}

	private Map<String, String> getFragmentParamMap(String fragment) {
		String[] params = fragment.split("&");
		Map<String, String> map = new HashMap<String, String>();
		for (String param : params) {
			String name = param.split("=")[0];
			String value = param.split("=")[1];
			map.put(name, value);
		}
		return map;
	}
}
